package xyz.lychee.lagfixer.libs.adventure.util;

import java.util.function.BooleanSupplier;
import xyz.lychee.lagfixer.libs.annotations.NotNull;
import xyz.lychee.lagfixer.libs.annotations.Nullable;

public enum TriState {
   NOT_SET,
   FALSE,
   TRUE;

   public @Nullable Boolean toBoolean() {
      switch (this) {
         case TRUE:
            return Boolean.TRUE;
         case FALSE:
            return Boolean.FALSE;
         default:
            return null;
      }
   }

   public boolean toBooleanOrElse(final boolean other) {
      switch (this) {
         case TRUE:
            return true;
         case FALSE:
            return false;
         default:
            return other;
      }
   }

   public boolean toBooleanOrElseGet(final @NotNull BooleanSupplier supplier) {
      switch (this) {
         case TRUE:
            return true;
         case FALSE:
            return false;
         default:
            return supplier.getAsBoolean();
      }
   }

   public static @NotNull TriState byBoolean(final boolean value) {
      return value ? TRUE : FALSE;
   }

   public static @NotNull TriState byBoolean(final @Nullable Boolean value) {
      return value == null ? NOT_SET : byBoolean(value);
   }
}
